package ch.epfl.cs108;

import java.io.PrintStream;
import java.util.List;

public interface ASCIImage {
    public int width();
    public int height();
    public List<String> drawing();

    public static ASCIImage fromString(String s) {
        return new FromString(s);
    }

    public static ASCIImage filled(int width, int height, char fill) {
        return new Filled(width, height, fill);
    }

    default public ASCIImage flippedHorizontally() {
        return new FlippedHorizontally(this);
    }

    default public ASCIImage transposed() {
        return new Transposed(this);
    }

    default public ASCIImage framed() {
        ASCIImage tb = fromString('+' + Strings.nCopies(width(), '-') + '+');
        ASCIImage lr = fromString(Strings.nCopies(height(), '|')).transposed();
        return tb.above(lr.leftOf(this.leftOf(lr)).above(tb));
    }

    default public ASCIImage leftOf(ASCIImage that) {
        return new LeftOf(this, that);
    }

    default public ASCIImage above(ASCIImage that) {
        return new Above(this, that);
    }

    default public void printOn(PrintStream s) {
    	for (String t: drawing())
    		s.println(t);
    }
}
